var theme = null;

slider.index = 0;
slider.references = new Array();
slider.event = function(index){
	slider.references[index].event();
}
slider.loadTheme = function(themeName){
	// include
	if(file_exists("Resources/"+themeName+"/def.js")){
		include("Resources/"+themeName+"/def.js");
	} else {
		log("Can't load theme \""+themeName+"\": file not found.");
		return 0;
	}
	/* redrawing instances */
	log("Redrawing "+slider.references.length+" instance(s).");
	for(var i in slider.references){
		slider.references[i].loadTheme();
		slider.references[i].redraw();
	}
	return 1;
}
/**
 * slider constructor
 * obj window = window to which the slider is to be attached
 */
function slider(window,themeSet){
	this.index = slider.index++;
	slider.references[this.index] = this;

	this.baseHandler = "slider.event("+this.index+");"

	/* public properties */
	this.showValue = false;
	this.stick = false;

	this.width = 100;
	this.vOffset = 0;
	this.hOffset = 0;
	this.min = 0;
	this.max = 100;
	this.value = 50;
	this.step = 1;
	this.unit = "";
	this.theme = themeSet;

	/* private properties */
	this.percent = 0.5;
	this.realWidth = this.width;
	this.callFromEvent=false;

	/* handlers */
	this.onMouseDown = null;
	this.onMouseUp = null;
	this.onMouseMove = null;
	this.onValueChanged = null;

	/* slider track left cap */
	this.sliderLeftCap = new Image(window);
	this.sliderLeftCap.src="Resources/null.png";
	//this.sliderLeftCap.src = theme.leftCapSrc;
	
	/* slider track right cap */
	this.sliderRightCap = new Image(window);
	this.sliderRightCap.src="Resources/null.png";
	//this.sliderRightCap.src = theme.rightCapSrc;
	
	/* slider track fill */
	this.sliderTrack = new Image(window);
	this.sliderTrack.src="Resources/null.png";
	//this.sliderTrack.src = theme.trackSrc;
	this.sliderTrack.onMouseDown = this.baseHandler;
	this.sliderTrack.onMouseUp = this.baseHandler;
	this.sliderTrack.onMouseMove = this.baseHandler;

	/* slider button */
	this.slider = new Image(window);
	this.slider.src="Resources/null.png";

	/* text field for displaying current value */
	this.val = new Text(window);
	this.val.bgOpacity = 0;
	this.val.data = "n/a";
	
	slider.loadTheme(this.theme);
	
	/* properties that change the object's position */
	this.watch("vOffset",this.setPos);
	this.watch("hOffset",this.setPos);
	this.watch("width",this.setWidth);

	/* properties that change the object's appearance */
	this.watch("min",this.setIntVal);
	this.watch("max",this.setIntVal);
	this.watch("unit",this.setIntVal);
	this.watch("showValue",this.setIntVal);
	this.watch("stick",this.setIntVal);
	this.watch("unit",this.setIntVal);
	this.watch("step",this.setIntVal);
	
	/* special properties */
	this.watch("theme",this.setTheme);
	this.watch("value",this.setValue);
	
	this.redraw();
	this.updateCursor();
}
slider.prototype.loadTheme = function(){
	log("Load theme "+theme.name+" for slider "+this.index+".");
	this.sliderLeftCap.src = theme.leftCapSrc;
	this.sliderRightCap.src = theme.rightCapSrc;
	this.sliderTrack.src = theme.trackSrc;
	this.slider.src = theme.sliderSrc;
	this.slider.hRegistrationPoint = theme.sliderWidth/2;
	this.slider.vRegistrationPoint = 0;
	this.val.font = theme.textFont;
	this.val.size = theme.textSize;
	this.val.color = theme.textColor;
}

/**
 * callbacks
 */
slider.prototype.setPos = function(prop,oldval,newval){
	this.redraw(prop,newval);
	return newval;
}
slider.prototype.setIntVal = function(prop,oldval,newval){
	this.value = this.calcValue(prop,newval);
	this.updateCursor(prop,newval);
	return newval;
}
slider.prototype.setValue = function(prop,oldval,newval){
	if(!this.callFromEvent){
		this.percent = (newval-this.min)/(this.max-this.min);
	}
	this.callFromEvent=false;
	var value = this.calcValue();
	this.updateCursor(prop,value);
	return value;
}
slider.prototype.setWidth = function(prop,oldval,newval){
	this.redraw(prop,newval);
	this.updateCursor();
	return newval;
}
slider.prototype.setStick = function(prop,oldval,newval){
	this.updateCursor(prop,newval);
	return newval;
}
slider.prototype.setTheme = function(prop,oldval,newval){
	if(slider.loadTheme(newval)){
		return newval;
	} else {
		return oldval;
	}
}

/**
 * 
 */
slider.prototype.redraw = function(prop,newval){
	/* copy local vars */
	var vOffset = this.vOffset;
	var hOffset = this.hOffset;
	var width = this.width;
	/* use updated value */
	if(undefined!=prop){
		//eval(prop+"='"+newval+"'");
		eval(prop+"="+newval);
	}
	this.realWidth = width-(theme.leftCapWidth+theme.rightCapWidth);
	
	/* left cap */
	this.sliderLeftCap.vOffset = vOffset;
	this.sliderLeftCap.hOffset = hOffset;

	/* fill track */
	this.sliderTrack.vOffset = vOffset;
	this.sliderTrack.hOffset = this.sliderLeftCap.hOffset+theme.leftCapWidth;
	this.sliderTrack.width = this.realWidth;

	/* right cap */
	this.sliderRightCap.vOffset = vOffset;
	this.sliderRightCap.hOffset = this.sliderTrack.hOffset+this.realWidth;
	
	/* slider button */
	this.slider.vOffset = this.sliderTrack.vOffset;
	this.slider.hOffset = this.sliderTrack.hOffset+this.percent*this.realWidth;
	
	/* text value */
	this.val.vOffset = vOffset+theme.height/2;
	this.val.hOffset = this.sliderRightCap.hOffset+theme.rightCapWidth+theme.textOffset;
}

slider.prototype.updateCursor = function(prop,newval){
	var min = this.min;
	var max = this.max;
	var step = this.step;
	var stick = this.stick;
	var unit = this.unit;
	var value = this.value;
	var showValue = this.showValue;
	/* use updated value */
	if(undefined!=prop){
		eval(prop+"='"+newval+"'");
	}
	var range = max-min;
	/* button position */
	if(this.stick){
		this.slider.hOffset = this.sliderTrack.hOffset+((value-min)/range*this.realWidth);
	} else {
		this.slider.hOffset = this.sliderTrack.hOffset+(this.percent*this.realWidth);
	}
	
	/* display value */
	if(showValue){
		this.val.data = "";//value+unit;
	} else {
		if(stick){
			this.val.data = "";//Math.round((value-min)/range*100)+"%";
		} else {
			this.val.data ="";// Math.round(this.percent*100)+"%";
		}
	}
}

slider.prototype.calcValue=function(prop,newval){
	var min = this.min;
	var max = this.max;
	var step = this.step;
	if(undefined!=prop){
		eval(prop+"='"+newval+"'");
		//eval(prop+"="+newval);
	}
	var value = roundToNearest(this.percent*(max-min)+min,step);
	return value;
}

slider.prototype.event = function(){
	var x = limit(system.event.x,0,this.realWidth);
	this.percent = x/this.realWidth;
	this.callFromEvent = true;
	this.value = this.calcValue();

	if(system.event.type=="mousemoved" && this.onMouseMove) this.onMouseMove(this.value);
	if(system.event.type=="mouseup" && this.onMouseUp) this.onMouseUp(this.value);
	if(system.event.type=="mousedown" && this.onMouseDown) this.onMouseDown(this.value);
}

function limit(n,min,max){
	if(n<min) return min;
	if(n>max) return max;
	return n;
}

function roundToNearest(X, Q) {
	return Q * Math.round(X / Q);
}

function file_exists(filename){
	var r;
	r = filesystem.itemExists(filename);
	return r;
}
